package lab.core 
{
	import as3isolib.display.IsoSprite;
	import as3isolib.display.IsoView;
	import as3isolib.display.primitive.IsoBox;
	import as3isolib.display.scene.IsoGrid;
	import as3isolib.display.scene.IsoScene;
	import as3isolib.enum.RenderStyleType;
	import as3isolib.graphics.SolidColorFill;
	import as3isolib.graphics.Stroke;
	import citrus.utils.objectmakers.tmx.TmxMap;
	import citrus.utils.objectmakers.tmx.TmxObject;
	import citrus.utils.objectmakers.tmx.TmxObjectGroup;
	import citrus.utils.objectmakers.tmx.TmxPropertySet;
	import citrus.utils.objectmakers.tmx.TmxTileSet;
	import feathers.controls.List;
	import feathers.controls.renderers.DefaultListItemRenderer;
	import lab.core.IsoLevel;
	import lab.data.Items;
	import lab.data.ItemVO;
	import lab.embed.Atlases;
	import lab.embed.Levels;
	import mvc.ui.editor.models.EditorItemVO;
	import mvc.ui.game.models.LevelModel;
	import org.osflash.signals.Signal;
	import starling.display.Image;
	import starling.events.EnterFrameEvent;
	import starling.textures.TextureAtlas;
	
	
	/**
	 * ...
	 * @author Roma Kraminskij - kraminskij@gmail.com
	 */
	public class IsoLevel extends IsoView
	{
		public static const CELL_SIZE:uint = 64;
		
		public static var instance				: IsoLevel;
		
		public var currentLevel					: int = 1;
		public var cosmo						: IsoSprite;
		public var grid							: IsoGrid;
		public var scene						: IsoScene;
		public var back							: IsoScene;
		public var items						: Vector.<ItemVO>;
		public var onLevelEnd					: Signal = new Signal();
		public var levelModel					: LevelModel;
		public var isNeedUpdate					: Boolean = false;
		public var sprites						: Vector.<IsoSprite> = new Vector.<IsoSprite>;
		public var cursor						: IsoBox = null;
		public var isEditorEnable				: Boolean = false;
		
		private var view						: IsoView;
		private var gridHolder					: IsoScene; 
		private var box							: IsoBox;
		private var atlas						: TextureAtlas;
		private var tileSet						: TmxTileSet;
		private var mapTiles					: Array;
		private var tmx							: TmxMap = null;
		private var isGridEnable				: Boolean = true;
		private var isRandomBG					: Boolean = false;
		
		private var mapObjects					: TmxObjectGroup = null;
		private var mapWidth:int = 0;
		private var mapHeight:int = 0;
		
		/*
		 * */
		public function IsoLevel( level:int = 1 ) 
		{
			IsoLevel.instance = this;
			
			currentLevel = level;
			
			super();
			
			scene = new IsoScene();
			
			back = new IsoScene();
			
			addScene( back );
			
			addScene( scene );
			
			if ( isGridEnable ) addDebugGrid();
			
			back.render();
			back.container.flatten();
			
			scene.render();
			
			//scene.container.x += Math.abs( bounds.x / 2 );
			//back.container.x += Math.abs( bounds.x / 2 );
			
			addEventListener(EnterFrameEvent.ENTER_FRAME, update );
		}
		
		public function setMap( xml:XML = null ):void 
		{
			mapWidth = xml.@width;
			mapHeight = xml.@height;
			
			levelModel.source = xml;
			
			items = new Vector.<ItemVO>;
			
			for each ( var i:XML in xml.item )
			{
				var i1:ItemVO = new ItemVO( i, this );
				i1.index = items.push( i1 );
			}
			
			for each ( var i0:ItemVO in items ) if ( i0.actor != null ) i0.actor.start( i0, this );
			
			levelModel.items = items;
			
			isNeedUpdate = true;
			//scene.container.x += Math.abs( bounds.x / 2 );
			scene.render();
		}
		
		
		public function addCursor( x:int, y:int ):void
		{
			var b:IsoBox = new IsoBox();
			b.setSize( IsoLevel.CELL_SIZE, IsoLevel.CELL_SIZE, 128 );
			b.moveTo( x * IsoLevel.CELL_SIZE, y * IsoLevel.CELL_SIZE, 10);
			
			var sc:SolidColorFill = new SolidColorFill(0xff0000, .2);
			var sf:Stroke = new Stroke(1, 0xFF0000, 0.5 );
			
			b.styleType = RenderStyleType.SHADED;
            b.fills = [ sc, sc, sc, sc, sc, sc ];
			b.stroke = sf;
			
			scene.addChild( b );
			
			b.container.touchable = false;
			
			cursor = b;
		}
		
		private function addRandomOuterIso( x:int, y:int ):void
		{
			var i:IsoSprite = new IsoSprite();
			
			var image:Image = new Image( Atlases.getAtlas("tiles_00").getTexture( "floor_00" ) );
			image.width = IsoLevel.CELL_SIZE * 2;
			image.height = IsoLevel.CELL_SIZE * 2;
			image.x = -IsoLevel.CELL_SIZE;
			image.y = -IsoLevel.CELL_SIZE;
			i.sprites = [ image ];
			
			i.moveTo( x * IsoLevel.CELL_SIZE, y * IsoLevel.CELL_SIZE, 0);
			i.setSize( IsoLevel.CELL_SIZE, IsoLevel.CELL_SIZE, IsoLevel.CELL_SIZE );
			
			back.addChild( i );
		}
		
		public function update( e:EnterFrameEvent = null ):void
		{
			if ( isNeedUpdate )	scene.render();
			// Update actors.
			for each ( var o:ItemVO in items ) if ( o.actor != null ) o.actor.update();
			// Flag for finished update cycle.
			isNeedUpdate = false;
		}
		/**
		 * 
		 */
		public function checkActors():void
		{
			for each ( var o:ItemVO in items ) if ( o.actor != null ) o.actor.check();
		}
		/**
		 * 
		 */
		private function updateActors():void
		{
			
		}
		/**
		 * 
		 * @param	x
		 * @param	y
		 * @return
		 */
		public function getItemsByXY( x0:int, y0:int ):Vector.<ItemVO>
		{
			var v:Vector.<ItemVO> = new Vector.<ItemVO>;
			
			for each ( var o:ItemVO in items ) 
			{
				if ( o.type.indexOf( Items.DOOR ) == 0 )
				{
					trace();
				}
				
				if ( x0 == o.x && y0 == o.y ) 
				{
					v.push( o );
				}
			}
			return v;
		}
		
		/**
		 * 
		 * @param	type
		 * @return
		 */
		public function isItemsHasTypeOf( type:String, i0:Vector.<ItemVO> ) : Boolean
		{
			for each ( var o:ItemVO in i0 ) if ( o.type.indexOf( type ) == 0 ) return true;
			return false;
		}
		
		/**
		 * 
		 * @param	type
		 * @return
		 */
		public function isItemsHasTypeOfXY( type:String, x:int, y:int ) : Boolean
		{
			var i0:Vector.<ItemVO> = getItemsByXY( x, y );
			for each ( var o:ItemVO in i0 ) if ( o.type.indexOf( type ) == 0 ) return true;
			return false;
		}
		/**
		 * 
		 * @param	type
		 * @return
		 */
		public function getItemTypeOfXY( type:String, x:int, y:int ) : ItemVO
		{
			var i0:Vector.<ItemVO> = getItemsByXY( x, y );
			for each ( var o:ItemVO in i0 ) if ( o.type.indexOf( type ) == 0 ) return o;
			return null;
		}
		/**
		 * Return first item given type.
		 * @param	type
		 * @return
		 */
		public function getItemByType( type:String ) : ItemVO
		{
			for each ( var o:ItemVO in items ) if ( o.type.indexOf( type ) == 0 ) return o;
			return null;
		}
		
		// FEATURE: Get the "g" key functional - show the grid from mediator.
		private function addDebugGrid():void
		{
			gridHolder = new IsoScene();
			
			grid = new IsoGrid();
			grid.stroke = new Stroke( 2, 0xff0000 );
			grid.cellSize = CELL_SIZE;
			
			grid.setGridSize( 12, 12, 0 );
			grid.container.flatten();
			grid.container.visible = false;
			
			gridHolder.addChild( grid );
			gridHolder.container.flatten();
			
			addScene( gridHolder );
			
			gridHolder.render();
		}
	}

}













































