package lab.data
{
	import as3isolib.core.IsoDisplayObject;
	import as3isolib.display.IsoSprite;
	import as3isolib.display.primitive.IsoBox;
	import as3isolib.enum.RenderStyleType;
	import as3isolib.graphics.SolidColorFill;
	import as3isolib.graphics.Stroke;
	import lab.actors.api.IActor;
	import lab.core.IsoLevel;
	import lab.data.Items;
	import lab.display.DataImage;
	import lab.embed.Atlases;
	import lab.registry.Reg;
	import nl.dpdk.utils.NumberUtils;
	import starling.textures.Texture;
	
	/**
	 * ...
	 * @author Roma Kraminskij - kraminskij@gmail.com
	 */
	public class ItemVO
	{
		private static var idCounter:int = 0;
		// Iso coords.
		public var x:int = 0;
		public var y:int = 0;
		// Unic id.
		public var id:int = -1;
		public var index:uint; // Index in array of IsoLevel.
		public var type:String = null;
		public var sprites:Vector.<IsoDisplayObject> = null;
		public var actor:IActor = null;
		public var source:XML = null;
		
		private var level:IsoLevel = null;
		
		public function ItemVO( source:XML, level:IsoLevel = null )
		{
			id = ++idCounter;
			
			this.source = source;
			type = source.@type;
			
			this.level = level;
			this.x = source.@x;
			this.y = source.@y;
			
			addIso();
		}
		
		public function changeType(type:String):void
		{
			this.type = type;
			// Remove old views.
			// level.scene.container.removeChild(sprites[0].sprites[0]);
			level.scene.removeChild(sprites[0]);
			deleteObj(sprites[0]);
			
			addIso();
		}
		/**
		 * 
		 */
		private function addIso(  ):void 
		{
			var actorClass:Class = Items.getActorByType( type );
			actor = ( actorClass != null ) ? new actorClass() : null;
			if ( actor != null ) return;
			
			var texture:Texture = Atlases.getAtlas( "tiles_00" ).getTexture( type );
			var image:DataImage = new DataImage( texture, this );
			
			var k:Number = texture.height / texture.width;
			
			// Random floor tile shifting.
			var xShift:Number = 0;
			var yShift:Number = 0;
			
			if ( type.indexOf ( Items.FLOOR ) == 0 )
			{
				var range:Number = 0.5;
				xShift = NumberUtils.getRandomNumberInRange( -range, range );
				yShift = NumberUtils.getRandomNumberInRange( -range, range );
			}
			
			image.width = IsoLevel.CELL_SIZE * 2;
			image.height = IsoLevel.CELL_SIZE * 2 * k;
			image.x = -IsoLevel.CELL_SIZE + xShift;
			image.y = -( ( IsoLevel.CELL_SIZE * 2 * k ) - IsoLevel.CELL_SIZE ) + yShift;
			image.smoothing = Reg.TEXTURE_SMOOTHING;
			
			var sprite:IsoSprite = new IsoSprite();
			
			sprite.name = type;
			sprite.sprites = [image];
			sprite.data = this;
			
			var h:Number = (type.indexOf( Items.WALL ) == 0 || type.indexOf( Items.BOX ) == 0 || 
				type.indexOf( Items.DOOR ) == 0 ) ? IsoLevel.CELL_SIZE : 1;
				
			var zs:Number = type.indexOf( Items.END ) == 0 ? 10 : -1; // Z coord.
			
			if ( type.indexOf( Items.BOX ) == 0 ) zs = 0;
			if ( type.indexOf( Items.DOOR ) == 0 ) zs = 0;
			
			sprite.setSize( IsoLevel.CELL_SIZE, IsoLevel.CELL_SIZE, h );
			sprite.moveTo( x*IsoLevel.CELL_SIZE-1, y*IsoLevel.CELL_SIZE-1, zs);
			
			level.scene.addChild(sprite);
			
			addObj(sprite);
		}
		
		
		public function getIsoByType(type:String):IsoSprite
		{
			for each (var o:IsoSprite in sprites)
				if (o.data != null && o.data.type == type)
					return o;
			
			return null;
		}
		
		public function addObj(obj:IsoDisplayObject = null):void
		{
			if (sprites == null)
				sprites = new Vector.<IsoDisplayObject>;
			
			sprites.push(obj);
		}
		
		public function deleteObj(obj:IsoDisplayObject = null):void
		{
			if (sprites == null)
				return;
			
			for (var i:int = 0; i < sprites.length; i++)
			{
				if (obj.UID == sprites[i].UID)
				{
					sprites.splice(i, 1);
					break;
				}
			}
		}
		
		public function toXML():XML
		{
			var xml:XML = new XML("<item />");
			xml.@type = type;
			xml.@x = x;
			xml.@y = y;
			
			if ( source.custom[0] != null ) xml.appendChild( source.custom[0] );
			
			return xml;
		}
		
	}

}
































