package mvc.ui.game.mediators {
	
	import as3isolib.core.IIsoDisplayObject;
	import as3isolib.geom.Pt;
	import flash.events.IEventDispatcher;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	import flash.ui.Keyboard;
	import lab.core.IsoLevel;
	import lab.data.Items;
	import lab.data.ItemVO;
	import lab.display.DataImage;
	import lab.registry.Reg;
	import mvc.events.AppEvent;
	import mvc.ui.editor.models.EditorModel;
	import mvc.ui.game.events.GameEvent;
	import mvc.ui.game.models.GameModel;
	import mvc.ui.game.models.LevelModel;
	import mvc.ui.game.views.GameView;
	import mvc.ui.menu.views.MenuView;
	import mvc.ui.starling.views.StarlingRoot;
	import org.gestouch.events.GestureEvent;
	import org.gestouch.gestures.PanGesture;
	import robotlegs.bender.extensions.starlingViewMap.impl.StarlingMediator;
	import starling.core.Starling;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	/**
	 * @author Roma Kraminskij - kraminskij@gmail.com
	 * 
	 * 
	 */
	public class GameViewMediator extends StarlingMediator
	{
		[Inject] public var dispatcher	: IEventDispatcher;
		[Inject] public var view		: GameView;
		[Inject] public var gameModel	: GameModel;
		[Inject] public var editorModel	: EditorModel;
		[Inject] public var levelModel	: LevelModel;
		
		static public const ZOOM_DELTA:Number = 0.1;
		static public const PAN_START_LIMIT:int = 20; 
		
		private var zoom:Number = 1;
		private var panPt:Point = new Point(); // Start pan mode point. Length in pixels for mouse after start down.
		private var panPt2:Point = new Point(); 
		private var hoverPt:Point = new Point();
		private var isPan:Boolean = false;
		private var zoomPoint:Point = new Point();
		private var currentItem:ItemVO;
		private var o:IIsoDisplayObject;
		
		public function GameViewMediator()
		{
		}

		override public function initialize():void
		{
			Starling.current.nativeStage.addEventListener( MouseEvent.MOUSE_WHEEL, viewZoom );
			Starling.current.nativeStage.addEventListener( KeyboardEvent.KEY_UP, keyHandler );
			
			dispatcher.addEventListener( AppEvent.REQUEST_MENU_VIEW, levelEnd );
			
			view.onLevelEnd.add( levelEnd );
			
			view.levelModel = levelModel;
			view.levelModel.isEditor = gameModel.isEditorMode;
			view.setMap( levelModel.source );
			
			if ( gameModel.isEditorMode ) view.addCursor(0, 0);
			// TODO: Move to viewport center.
			var k:Number = Reg.GAME_WIDTH / view.scene.container.width;
			view.scaleX = view.scaleY = k;
			//view.scene.container.scaleX = view.scene.container.scaleY = k;
			//view.back.container.scaleX = view.back.container.scaleY = k;
			
			var pan:PanGesture = new PanGesture( view );
			pan.addEventListener( GestureEvent.GESTURE_BEGAN, onPanStart );
			pan.addEventListener( GestureEvent.GESTURE_CHANGED, onPanChange );
			pan.addEventListener( GestureEvent.GESTURE_ENDED, onPanEnd );
			
			view.scene.container.addEventListener( TouchEvent.TOUCH, touchHandler );
			
			//view.panTo( 0, 0 );
			//view.x = 0;
			//view.y = 0;
			//view.centerOnPt( new Pt(6*IsoLevel.CELL_SIZE, 6*IsoLevel.CELL_SIZE, 0) );
			
			o = view.getItemsByXY( 6, 6 )[0].sprites[0] as IIsoDisplayObject;
			view.centerOnIso( o );
			//view.renderISO();
			//view.centerOnPt( new Pt( 6*IsoLevel.CELL_SIZE, 6*IsoLevel.CELL_SIZE, 0 ), false );
			
		}
		
		private function touchHandler( e : TouchEvent ) : void 
		{
			var t:Touch = e.getTouch( view.scene.container );
			
			if ( t != null )
			{
				if ( gameModel.isEditorMode )
				{
					// Hover mouse.
					if ( t.phase == TouchPhase.HOVER )
					{
						if ( e.target is DataImage )
						{
							if ( (e.target as DataImage).item != null && (e.target as DataImage).item is ItemVO )
							{
								view.cursor.moveTo( (e.target as DataImage).item.x * IsoLevel.CELL_SIZE, 
									(e.target as DataImage).item.y * IsoLevel.CELL_SIZE, -20 );
								
								currentItem = (e.target as DataImage).item;
								view.scene.render();
							}
						}
					}
					// Click / Touch.
					if ( t.phase == TouchPhase.ENDED )
					{
						if ( (e.target is DataImage) &&
							 (e.target as DataImage).item != null && (e.target as DataImage).item is ItemVO )
						{
							view.cursor.moveTo( (e.target as DataImage).item.x * IsoLevel.CELL_SIZE, 
									(e.target as DataImage).item.y * IsoLevel.CELL_SIZE, 10 );
							
							currentItem = (e.target as DataImage).item;
							
							if ( (e.target as DataImage).item.type.indexOf( Items.FLOOR ) == 0 && 
								(   editorModel.currentItem.type.indexOf( Items.PLAYER ) == 0 ||
									editorModel.currentItem.type.indexOf( Items.END ) == 0 ||
									editorModel.currentItem.type.indexOf( Items.AUTO_DOOR ) == 0 || 
									editorModel.currentItem.type.indexOf( Items.BOX ) == 0 || 
									editorModel.currentItem.type.indexOf( Items.DOOR ) == 0) )
							{
								var xml:XML = <item x={currentItem.x} y={currentItem.y} type={editorModel.currentItem.type}/>;
								view.items.push( new ItemVO( xml, view ) );
								view.scene.render();	
							}
							else
							{
								currentItem.changeType( editorModel.currentItem.type );
								view.scene.render();
							}
						}
					}
				}
			}
		}
		
		private function onPanChange(e:GestureEvent):void 
		{
			if ( isPan )
			{
				panPt.setTo( (e.target as PanGesture).location.x, (e.target as PanGesture).location.y );
				
				view.panBy( panPt2.x - panPt.x, panPt2.y - panPt.y );
				
				panPt2.setTo( panPt.x, panPt.y);
			}
		}
		
		private function onPanStart(e:GestureEvent):void 
		{
			isPan = true; 
			panPt = (e.target as PanGesture).location.clone();
			panPt2 = panPt.clone();
		}
		private function onPanEnd(e:GestureEvent):void 
		{
			isPan = false; 
		}
		
		override public function destroy():void
		{
		}
		
		private function keyHandler(e:KeyboardEvent=null):void 
		{
			if ( e.keyCode == Keyboard.G ) view.grid.container.visible = !view.grid.container.visible; // TODO: Check the G key functional.
			if ( e.keyCode == Keyboard.ESCAPE ) levelEnd();
		}
		
		private function levelEnd( e:Object = null ):void
		{
			view.removeFromParent( true );
			(Starling.current.root as StarlingRoot).addChild( new MenuView() );
			dispatcher.dispatchEvent( new GameEvent( GameEvent.LEVEL_ENDED ) );
		}
		
		
		// FIXME: Divide wheel action in editor and game level.
		private function viewZoom(e:MouseEvent) : void
		{
			/*zoomPoint.setTo( e.stageX, e.stageY );
			 = Starling.current.stage.hitTest( zoomPoint );
			var o:DisplayObject = view.hitTest( zoomPoint );*/
			
			if (e.delta > 0) zoom += ZOOM_DELTA;
			if (e.delta < 0) zoom -=  ZOOM_DELTA;
			
			if (zoom < 0) zoom = 0;
			// TODO: Set pivot view to center.
			view.currentZoom = zoom;
		}
	}
}









































