package mvc.ui.root.commands 
{
	import citrus.sounds.SoundManager;
	import feathers.themes.MetalWorksMobileTheme;
	import flash.events.DataEvent;
	import flash.events.Event;
	import flash.events.IEventDispatcher;
	import flash.system.Capabilities;
	import lab.embed.Atlases;
	import lab.embed.Sounds;
	import mvc.services.fullscreen.FullscreenService;
	import mvc.ui.console.view.Console;
	import mvc.ui.game.models.GameModel;
	import mvc.ui.menu.views.MenuView;
	import mvc.ui.root.events.RootViewEvent;
	import mvc.ui.root.models.AppModel;
	import mvc.ui.starling.views.StarlingRoot;
	import mvc.ui.toolspanel.views.ToolsView;
	import nl.dpdk.commands.tasks.CallBackTask;
	import nl.dpdk.commands.tasks.CompositeTask;
	import nl.dpdk.commands.tasks.Sequence;
	import nl.dpdk.commands.tasks.SequenceEvent;
	import robotlegs.bender.extensions.commandCenter.api.ICommand;
	import robotlegs.bender.extensions.viewManager.impl.ViewManager;
	import robotlegs.bender.framework.api.ILogger;
	import starling.core.Starling;
	import lab.embed.Atlases;
	import lab.embed.Sounds;
	import lab.registry.Reg;
	
	/**
	 * ...
	 * @author Roma Kraminskij - kraminskij@gmail.com
	 */
	public class AppStartupCommand implements ICommand 
	{
		[Inject] public var event:RootViewEvent;
		[Inject] public var appModel:AppModel;
		[Inject] public var gameModel:GameModel;
		[Inject] public var dispatcher:IEventDispatcher;
		[Inject] public var logger:ILogger;
		[Inject] public var fullscreenService:FullscreenService;
		
		private var seq:Sequence;
		
		public function AppStartupCommand()
		{
			
		}
		
		/* INTERFACE robotlegs.bender.extensions.commandCenter.api.ICommand */
		
		public function execute():void 
		{
			Atlases.instance = new Atlases();
			
			fullscreenService.init();
			
			for ( var i:int = 0; i < Sounds._temp.length; i++ )
			{
				var id:String = new String( Sounds._temp[i].id );
				delete Sounds._temp[i].id;
				var params:* = Sounds._temp[i];
				SoundManager.getInstance().addSound( id, params );
			}
			
			SoundManager.getInstance().playSound("lightlistening_loop");
			
			new feathers.themes.MetalWorksMobileTheme(  null, false ); 
			
			seq = new Sequence( true );
			
			seq.add( new CallBackTask( appInfoLog ) );
			seq.add( new CallBackTask( addMVCViews ) );
			
			seq.addEventListener( SequenceEvent.DONE, doneHandler );
			seq.addEventListener( SequenceEvent.ERROR, errorHandler );
			
			seq.execute();
		}
		
		private function appInfoLog():void 
		{
			var ver:String = "ver. " + Version.Major.toString() + "." + Version.Minor.toString() + " build " + Version.Build;
			var date:String = Version.Timestamp;
			var author:String = Version.Author;
			
			Console.log( ver + " - " + date );
			Console.log( "Author: " + author );
			Console.log( "system: " + Capabilities.os );
			
			Console.log("Keys: '~':Console,'g':Grid, 'ESC':Menu");
		}
		
		private function addMVCViews():void
		{
			with ( Starling.current.root as StarlingRoot )
			{
				addChild( new MenuView() );
				addChild( new Console() );
				addChild( new ToolsView() );
			}
		}
		
		private function doneHandler(e:SequenceEvent):void 
		{
			seq.removeEventListener( "DONE", doneHandler );
			seq.removeEventListener( "ERROR", errorHandler );
		}
		
		private function errorHandler(e:SequenceEvent):void 
		{
			logger.error( "Main load app sequence error!" + e.getMessage() );
		}

		
	}

}